@@ -34,10 +34,13 @@ setup(
     packages=["lcitool"],
     scripts=["bin/lcitool"],
     package_data={
-        "lcitool": get_recursive_datafiles(["lcitool/etc",
-                                            "lcitool/configs/install",
+        "lcitool": get_recursive_datafiles(["lcitool/ansible",
+                                            "lcitool/configs",
+                                            "lcitool/containers",
                                             "lcitool/cross",
-                                            "lcitool/ansible"]),
+                                            "lcitool/etc",
+                                            "lcitool/facts",
+                                            "lcitool/install", ]),
     },
     author="libvirt team",
     author_email="libvir-list@redhat.com",
